<h1 id="Top"> Getting started with EB GUIDE Cluster Demo</h1> 

The EB GUIDE Cluster Demo showcases a cluster interface modeled with EB GUIDE Studio. The demo consists of two models:
- the <varname>cluster model</varname> for the customer interface as seen in the picture below
- the <varname>injection model</varname> needed for testing, demonstrating and customizing the cluster model

With this Demo model you can learn how to model your own cluster interface and it helps you to understand the concepts for modeling with EB GUIDE. 


![image of the cluster HMI](./Cluster/resources/master/main.png)

The following chapters explain how to set up and work with the EB GUIDE Cluster Demo and provide background information about the most important concepts:

* <a href="#Installation">Installation</a>  
  * <a href= "#Install_GUIDE">Installing EB GUIDE Studio</a>
  * <a href= "#Install_Git">Installing Git and cloning the repository</a>

* <a href="#Working">Working with the EB GUIDE Cluster Demo</a>
  * <a href="#Cluster_model">Cluster model</a>
  * <a href="#Injection_model">Injection model and model interface</a>
  * <a href="#Multi_model">Multi model setup for interaction between Cluster and Injection model</a>
  * <a href="#Navigating_model">Navigating through the cluster model</a>
  * <a href="#Modeling_behavior">Modeling the behavior</a>
  * <a href="#Monitor">Working with EB GUIDE Monitor</a>
  * <a href="#File_structure">File structure of the Cluster model</a>
  * <a href="#Naming">Naming conventions</a>
  * <a href="#Configuration">Configuration</a>
  
* <a href="#Background">Background information for EB GUIDE Studio</a>
  * <a href="#States">States and transitions</a>
  * <a href="#Appearance">Modeling HMI appearance</a>
  * <a href="#Datapool">Datapool items</a>
  * <a href="#Animations">Animations</a>
  * <a href="#Skin_Language">Skin and language support</a>
  * <a href="#Namespaces">Namespaces</a>
  * <a href="#Scripting">Scripting</a>



<h1 id="Installation">Installation</h1>

<h2 id="Install_GUIDE">Installing EB GUIDE Studio</h2>  

1. You can download and install the EB GUIDE Community Edition for free or purchase the Enterprise Edition from https://www.elektrobit.com/ebguide/try-eb-guide/.  

2. Follow the installation instructions using the default settings. If the settings are customized, you need to modify settings later in the scripts.

<h2 id="Install_Git">Installing Git and cloning the repository</h2>  

1. Download and install Git from https://git-scm.com/downloads or a Git client like GitHub Desktop (https://desktop.github.com/).  

2. Clone the repository to a folder of your choice with the following command:

        $ git clone <url>

<a href="#Top">Back to top</a>

<h1 id="Working">Working with the EB GUIDE Cluster Demo</h1>

The following instructions guide you through setting up the <varname>cluster</varname> and <varname>injection</varname> model, the interaction between both models and how to modify the behavior.

**Prerequisite:**

* EB GUIDE Studio 6.10 is installed at ``"C:\Program Files\Elektrobit\EB GUIDE 6.10\"`` or change the variable ``EB_GUIDE_FOLDER`` in the script to your installation location.

<h2 id="Cluster_model">Cluster model</h2>  
The cluster model is the main model in this project. It consists of the gauges for the speed, power, charge and range as well as applications like navigation and tire pressure monitor. 

### Running the cluster model  

1. Start EB GUIDE Studio.
2. To open the cluster model open ``\Cluster\Cluster.ebguide``.
3. To start the cluster simulation press ``F5`` or in the command area click ``▷``.  
4. Repeat the steps above for the <a href="#Injection_model">Injection model</a>: ``\Injection\Injection.ebguide``.

<h2 id="Injection_model">Injection model and model interface</h1>

Next to the cluster model this project contains another model called `injection model`. The injection model is used for demo and testing purposes and offers possibilities to configure the cluster model. It can change values like speed or charge state as well as skin, language and units in the cluster model. The two models communicate via Inter-process communication (IPC) and share a common model interface. It is possible to add datapool items and events to these interfaces. As an example the namespace `Cluster:MIF_Injection` contains all datapool items that are tagged for the model interface between the cluster and injection model. The interface can be exported and imported in the `PROJECT CENTER` under `CONFIGURE`.


<h2 id="Multi_model">Multi model setup for interaction between Cluster and Injection model</h2>  

The multi model setup uses IPC (Inter process communication) for interaction between the cluster and the injection model. Therefor, booth models will be started in separate processes while the cluster model is in the role of the server and the injection model the role of the client.

#### Instruction

* To export and start the models execute the batch file ``\scripts\start_multi_model.bat``.

<h2 id="Navigating_model">Navigating through the cluster model</h2>  

To navigate through the pages (navigation, tire pressure, suspension, headlights and drive) of the cluster use the following keys:<br>
  - 'Cursor ⭢' or 'Num 6' to go forwards
  - 'Cursor ⭠' or 'Num 4' to go backwards

Additonal features are available for the pages **Suspension** and **Headlights**:
  - 'Cursor ⭡' or 'Num 8' to increase a value
  - 'Cursor ⭣' or 'Num 2' to decrease a value

 <h2 id="Modeling_behavior">Modeling the behavior</h2>  

This model consists of several state machines:  
+ `SM_Main`: The base state machine. It is executed when the application starts and will be terminated when the application stops. `SM_Main` includes the background image only. Other components are composed via dynamic state machine and will be added as layers above the `SM_Main` visually and as stack entries logically. The dynamic state machines are pushed to the stack as entry action of the SM_Main, e.g. `f:pushDynamicStateMachine(popup_stack:SM_Main, sm:SM_Powermeter, 0)`. 

+ `SM_Speedometer`: contains the speedometer and the additional instrument for the range state.

+ `SM_Powermeter`: contains the power meter without its center content `SM_Powermeter_Content`, and the additional instrument for the charge state.

+ `SM_Center`: contains information about navigation, tire pressure, suspension and headlight. 

+ `SM_Powermeter_Content`: contains additional information that will be shown synchronously to the content of `SM_Center`. Furthermore the user can change settings of the content like *suspension height*.

+ `SM_Trip_Bar`: contains trip data like current or total trip distance, and a clock.

Each state machine is decoupled from the others and runs in its own process. Communication and synchronization between state machines is done via datapool items and events. Therefore state machines are a good architectural pattern to split the application down into smaller manageable modules.

<h2 id="Monitor">Working with EB GUIDE Monitor</h1>

Everytime the simulation starts, EB GUIDE Monitor starts as well. With this tool it is possible to interact with the model. EB GUIDE monitor supports firing events, reading and writing datapool items, displaying the current state of all running state machines and the tracing massages.

<h2 id="File_structure">File structure of the Cluster model</h2>

The table shows the file structure on the basis of the Cluster model.

| Heading       | Path                                | Comment                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       |
| ------------- | ----------------------------------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| Models    | `Cluster`<br /> `Injection`         | The model defintions can be found here. The files are stored in a format called "gdata". This is an internal JSON like human readable format.<br />- [Cluster.ebguide](Cluster/Cluster.ebguide) and [Injection.ebguide](Injection/Injection.ebguide) are the main files of the project.<br />- **datapool.gdata** contains all datapool defintions of the cluster<br />- **events.gdata** strores the used events<br />- **languages.gdata** is used for the translation. |
| Profiles      | `Cluster\configuration\`            | - Edit (internally used by EB GUIDE Studio)<br />- Simualtion (the default profile that is used for preview in EB GUIDE Studio)<br />- Target (special target customized)                                                                                                                                                                                                                                                                                                                     |
| Resources     | `Cluster\resources`                 | All used assets can be found here.<br />                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| Fonts         | `Cluster\resources\fonts`           | - **Barlow** is used as main font<br />- **Noto_Sans** is used for asian characters.<br />                                                                                                                                                                                                                                                                                                                                                                                                |
| Images        | `Cluster\resources\images`          | - Used for graphical assets<br />                                                                                                                                                                                                                                                                                                                                                                                                                                                    |
| Master Images | `Cluster\resources\master`          | - Design templates for verification use cases<br />- These graphics are not used directly for the cluster model                                                                                                                                                                                                                                                                                                                                                                               |
| Interfaces    | `Cluster\resources\modelinterfaces` | The model interfaces.                                                                                                                                                                                                                                                                                                                                                                                                                                                                         |
| Namespaces    | `Cluster\resources\namespaces`      | Namespace definitions can be found here.                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| Statemachine  | `Cluster\resources\statemachines`   | The state machines can be found here.     |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| Templates     | `Cluster\resources\templates`       | The template can be found here. |                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               |
| VTAS          | `Cluster\resources\vtas`            | VTA stands for View Transaction Scripts.                                                                                                                                                                                                                                                                                                                                                                                                                                                      |
| Interfaces    | `interface`                         | The interfaces are used to share data:<br />- Injection Model ->**Cluster_MIF_Injection.json**<br />- Car Data -> **Cluster_MIF_Plugin_CarInfo.json**                                                                                                                                                                                                                                                                                                                                          |
| Scripts       | `scripts`                           | Scripts are helpful to automate complex processes.<br />**start_multi_model.bat** will automate:<br />- Export the Cluster model<br />- Export the Injection model<br />- Copy the datapool items as resources (needed to communicate between the models)<br />- Start the 2 models with GtfStartup.exe<br />- Start the EB GUIDE Monitor as well<br />**stop_all.bat** will close all EB GUIDE TF processes.                                                                                         |
| Export        | `export`                            | The Export directory is generated and contains the exported model files. <br />To generate content into this directory, start the script `scripts\start_multi_model.bat`.                                                                                                                                                                                                                                                                                                                     |

<a href="#Top">Back to top</a>

<h2 id="Naming">Naming conventions</h1>

This model uses snake_case as base for naming convention. Some names are using prefixes:

- SM: for state machines
- S: for states
- V: for views
- T: for templates
- MIF: for model interfaces

In the cluster demo model, the name `SM_Center` is used for the state machine named *center*. If a prefix is used, each word starts with an upper case letter in the name. Events, datapool items and instances of widgets in the widget trees start with a lower case letter, e.g. `speed_value`.

<h2 id="Configuration">Configuration</h1>

In the `Project center` the tab `CONFIGURE` offers the possibility to manage languages, skins, model interfaces and profiles.  

EB GUIDE uses profiles to configure the project for it's target platform. It comes default with the edit and simulation profile. The edit profile is used for the rendered content in the editor. The simulation profile is used for the simulation.  

Each profile has three tabs named *Scenes*, *Platform* and *Model*: 

+ In the *Scenes* tab you can change e.g. width and height of the scene or enable anti-aliasing.  

+ The *Platform* tab manages the settings for runtime environment like which plugins to load. In the cluster demo model the IPC plugin is loaded and configured:

``` json
{
  "gtf": {
    "core": {
      "pluginstoload": [
        "GtfService",
        "GtfIpc"
      ]
    },
    "ipc": {
      "role": "server",
      "discovery": {
        "mode": "direct",
        "network": "127.0.0.1"
      },
```

+ The *Model* tab configures model specific settings. The following example shows  how to define default settings. These values are read by the `script init_default_settings` and written into datapool items. The table below explains the possible settings.

``` json
{
  "default_settings" : {
    "language" : "en-GB",
    "skin" : 0,
    "units" : {
      "am/pm" : false,
      "speed" : "km/h",
      "distance" : "km",
      "distance_2" : "m"
    },
    "demo_mode" : true,
    "show_logo" : false
  }  
}
```
The following table explains the possible settings for the example above:


| json object     | description                                                                                                                                                |
| --------------- | ---------------------------------------------------------------------------------------------------------------------------------------------------------- |
| language        | ISO language code 'language-COUNTRY'.<br>List of supported language codes.<br>String:<br><ul><li>en-GB</li><li>de-DE</li><li>fr-FR</li><li>zh-CN</li></ul> |
| skin            | Selected skin.<br>Integer[0..3]:<br><ul><li>0 = "Standard Skin"</li><li>1 = "Skin_1_blue"</li><li>2 = "Skin_1_red"</li><li>3 = "Skin_1_silver"</li></ul>   |
| units_am/pm     | HMI display setting for Time.<br>Boolean:<br><ul><li>true = 12 hour time format with am/pm</li><li>false = 24 hour clock</li></ul>                         |
| units_speed     | HMI display setting for speed.<br>String:<br><ul><li>km/h</li><li>mph</li></ul>                                                                            |
| units_distance  | HMI display setting for distances.<br>String:<br><ul><li>km</li><li>mi</li></ul>                                                                           |
| units_distance2 | HMI display setting for other distances. <br>String:<br><ul><li>m</li><li>ft</li></ul>                                                                   |
| demo_mode       | HMI Cluster setting for demonstration mode.<br>Boolean:<br><ul><li>true = demo mode is enabled (simulates driving)</li><li>false = disabled</li></ul>      |
| show_logo       | HMI Cluster setting for EB logo.<br>Boolean:<br><ul><li>true = enabled</li><li>false = disabled</li></ul>                                                  |

<a href="#Top">Back to top</a>

<h1 id="Background">Background information for EB GUIDE Studio</h1>

With EB GUIDE Studio it is possible to model the behavior and appearance of an application. The behavior will be modeled using state machines while the appearance modelling is using views, widgets, animations. These and other building blocks are described below using the cluster model as an example.

*This is a brief introduction how to use and understand the model. For more information, see the [EB GUIDE user guide](https://www.elektrobit.com/ebguide/documentation/), or press `F1` in EB GUIDE Studio.*  

<h2 id="States">States</h2>

States are bound to one state machine. EB GUIDE Studio supports initial and final states to define where the application flow starts and terminates for the specific state machine.  
Furthermore there are compound states that are groups of other states. They can be collapsed in the editor and help to organize big state machines.  
View states are the connection point between the behaviour and the appearance of the application. They have a view widget embedded where it is possible to define the appearance of the application.  
Last, there are choice states where the application workflow can be split into different paths depending on the condition.

<h3 id="State_actions">State actions</h2>

EB GUIDE supports entry and exit actions that are called when a state is entered or exited. Everything that could be expressed with a script function could be executed here.  
The cluster demo model uses entry actions to synchronize `SM_Center` and `SM_Powermeter_Content`. If a view state in `SM_Center` is entered, the entry action will fire an event to set `SM_Powermeter_Content` to its corresponding state.   

<h3 id="Transitions">Transitions and events</h2>

Transitions are the connections between states. They define the application flow using triggers and conditions. Default transitions will be traversed without a condition.  
In the cluster demo model, the connection between the initial state and the state `S_Main` will directly route the application to state `S_Main`. Other transition will be traversed if a given trigger is executed.  

Events are triggers in EB GUIDE. If an event is thrown into a machine, an interpreter will try to find a transition that consumes this event as a trigger starting at the current state. If such a transition exists the application flow will proceed to the next state that is the target of the transition. The cluster demo model uses this mechanism to change the content of the center by firing an event every time an arrow key is pressed (e.g. `HK_RIGHT`).  
Furthermore, it is possible to add a condition to the transition. This is a guard that lets the transition be executed if the corresponding condition evaluates true.

<a href="#Top">Back to top</a>

<h2 id="Appearance">Modeling HMI appearance</h2>

The appearance is modeled with views, widgets, templates, properties, datapool item and animations.

<h3 id="Views">Views</h3>

A view can be seen as a canvas where objects can be drawn to. These objects are called basic widgets and templates in EB GUIDE and they are organized in a widget tree. The tree is ordered. Widgets at the bottom of the tree will be drawn above widgets at the top of the tree. Views have properties for position (`x` and `y`) and dimension (`width` and `height`) per default.

<h3 id="Widgets">Widgets</h3>

EB GUIDE offers basic widgets like images, rectangle, ellipses, labels and alpha masks. They can be used on their own or as a composition of them in templates.

<h3 id="Templates">Templates</h3>

Templates are compositions of basic widgets and other templates. There are two main use-cases:  

+ They can be used as a constant component like `T_Speedometer` in this model. It helps to decouple widget compositions from the view structure, even if it is used only once as in this model.  

+ They can be used as configurable components with different values for their interface properties for each instance. An example is `T_Custom_Label` in this model. It is basically a widget that consists of a value and a unit. The template is responsible for the consistent layout and style in the whole model, but it can be customized if necessary. The template hides its internal structure and logic, and only exposes properties like the actual value and unit to the outside.  

<a href="#Top">Back to top</a>
  
<h3 id="Widget_properties">Widget properties and widget features</h3> 

Each widget has a set of properties that can be seen as an interface to adapt the widget itself. For example, each basic widget has properties for visibility, x, y, width and height, while a label has additional properties for text and font. Widget features extend these properties.  
For buttons it is common to add Widget feature properties like `touch released` where it is possible to add a script that will be called when the button was pressed.  
Furthermore it is possible to link a property of a widget to the property of the same type of another widget. If the value of a property changes, all linked properties get updated. For example, a widget's width linked to its parent's width, will result in the same width for both widgets no matter which of them is changed. Links to other widget properties have a yellow mark, links to datapool items have an orange mark.

<a href="#Top">Back to top</a>

<h2 id="Datapool">Datapool items</h3>

Datapool items can be seen as typed global properties. For the purpose of reuse, widget properties can be linked to datapool items.  
Datapool items can be accessed from outside the model, e.g. from application code or the EB GUIDE Monitor. In the demo cluster model, datapool items are used for the information on current speed `Cluster:Values:Vehicle:speed_value` or color `Cluster:Style:Tube:color_tube_needle`.

<a href="#Top">Back to top</a>

<h2 id="Animations">Animations</h3>

Animations are realized by the animation widget as part of the widget tree or as a view transition animation (VTA).  
In the cluster model a VTA is used for the view template `T_View_Center`. The animation describes how the visual change for instances of template `T_View_Center` should look like, e.g. from `map` view to `tire pressure` view.  

Inside an animation, values for widget properties and datapool items are calculated using configurable curves. Each type of curve provides general properties like duration and delay as well as specific properties like a start value and end value or a frequency and amplitude. If an animation is played the value for each curve is calculated based on the current animation time. In the cluster demo model, the opacity and the vertical translation of view and statusbar are animated so that the source view is faded out to full transparency and the target view is faded in from full transparency while the views are also moved.

<a href="#Top">Back to top</a>

<h2 id="Skin_Language">Skin and language support</h3>

Datapool items can be tagged to support skinning and languages. Therefore skins and languages must be defined in the configuration page of the model.  

---
**Example:**

The datapool item `Cluster:Style:Tube:color_tube_needle` has different colors for each skin and `Cluster:Values:Vehicle:suspension_niveaus` has different translations for each language.

---

Skins and languages can be changed in the editor or the EB GUIDE Monitor at runtime or via a script call.

<a href="#Top">Back to top</a>

<h2 id="Namespaces">Namespaces</h2>

Namespaces help to organize the model by grouping datapool items and events. The cluster demo model uses the following namespace structure:

- **MIF_***: for imported namespaces from model interfaces
- **Settings**: contains all configuration items like the display height, configured distance unit or available languages 
- **Style**: contains all style information like fonts, colors or position information and has sub-namespaces like *Animation*, *Center* or *Tube*
- **Units**: contains all unit information, like speed or distance
- **Values**: contains all external values like vehicle data or current media title 

<a href="#Top">Back to top</a>

<h2 id="Scripting">Scripting</h2>

EB GUIDE Script is the built-in script language of EB GUIDE. It is used to fire events, write datapool items or modify widget properties. As an example, the script for calculating the rotation of the needle in the speedometer `T_Speedometer.needle.rotate()`:

```
function(v:arg0::bool)
{
	v:this.rotation = - v:this->^.central_angle * v:this->^.speed / v:this->^.speed_max
}
```
The rotation angle of the needle relative to its zero value is the ratio of the current speed and the maximum speed multiplied by the central angle of the speedometer. Since the needle rotates to the right for higher speed values the rotation angle must be negative. As short introduction the syntax, `v` is the namespace for local variables while `v:this` refers to the current widget and `->^` is the reference to its parent widget. So `v:this->^.central_angle` is the reference to the parent's property `central_angle`.

For more details about the scripting language refer to the [EB GUIDE user guide](https://www.elektrobit.com/ebguide/documentation/).

<a href="#Top">Back to top</a>


